/* (C) 2021 YiRing, Inc. */
package com.yiring.app.param;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serializable;
import java.util.Objects;
import javax.validation.constraints.DecimalMin;
import javax.validation.constraints.NotNull;
import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.FieldDefaults;
import lombok.experimental.SuperBuilder;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;

/**
 * 分页查询参数公共父类
 *
 * @author ifzm
 * @version 0.1 2019/3/10 16:29
 */
@SuppressWarnings({ "unused" })
@ApiModel
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@FieldDefaults(level = AccessLevel.PRIVATE)
public class BasicPaginationParam implements Serializable {

    private static final long serialVersionUID = 6103761701912769946L;

    @ApiModelProperty(value = "分页条数", example = "10", required = true)
    @NotNull(message = "分页条数不能为空")
    @DecimalMin(value = "1", message = "分页条数不能小于1")
    Integer pageSize;

    @ApiModelProperty(value = "当前页数", example = "1", required = true)
    @NotNull(message = "当前页数不能为空")
    @DecimalMin(value = "1", message = "当前页数不能小于1")
    Integer pageNo;

    @ApiModelProperty(value = "排序参数", example = "id")
    String sortField;

    @ApiModelProperty(value = "排序方向（ASC|DESC）", example = "DESC")
    Sort.Direction sortOrder;

    /**
     * 根据参数构建分页对象
     *
     * @param paginationParam {@link BasicPaginationParam}
     * @return Pageable
     */
    public static Pageable toPageable(BasicPaginationParam paginationParam) {
        if (paginationParam == null) {
            return Pageable.unpaged();
        }

        Sort sort = Sort.unsorted();
        if (Objects.nonNull(paginationParam.getSortField())) {
            sort = Sort.by(new Sort.Order(paginationParam.getSortOrder(), paginationParam.getSortField()));
        }
        return PageRequest.of(paginationParam.getPageNo() - 1, paginationParam.getPageSize(), sort);
    }
}
